#ifndef FSTACK_HPP
#define FSTACK_HPP

template <typename T>
class FStack {
private:
  T* data;
  unsigned int m_size;
  unsigned int m_capacity;
public:
  FStack();
  FStack(const unsigned int capacity);
  FStack(const FStack<T>& other);
  virtual ~FStack ();

  FStack<T>& operator=(const FStack<T>& other);

  T& operator[](unsigned int index);
  const T& operator[](unsigned int index) const;
  unsigned int size() const;
  unsigned int capacity() const;

  void push(const T& value);
  void pop();
  T& back();
  const T& back() const;
};

template <typename T>
FStack<T>::FStack() {
  data = new T[2];
  m_size = 0;
  m_capacity = 0;
}

template <typename T>
FStack<T>::FStack(const unsigned int capacity):m_size(0), m_capacity(capacity) {
  data = new T[m_capacity];
}

template <typename T>
FStack<T>::FStack(const FStack<T>& other) :
  m_size(other.m_size),
  m_capacity(other.m_capacity) {
  data = new T[m_capacity];

  for (unsigned int i = 0; i < m_size; i++) {
    data[i] = other.data[i];
  }
}

template <typename T>
FStack<T>::~FStack() {
  delete[] data;
}

template <typename T>
FStack<T>& FStack<T>::operator=(const FStack<T>& other) {
  if (this != &other) {
    T* nStack = new T[other.m_capacity];
    for (unsigned int i = 0; i < other.m_size; i++) {
      nStack[i] = other.data[i];
    }

    delete[] data;

    data = nStack;
    m_size = other.m_size;
    m_capacity = other.m_capacity;
  }

  return *this;
}

template <typename T>
T& FStack<T>::operator[](unsigned int index) {
  return data[index];
}

template <typename T>
const T& FStack<T>::operator[](unsigned int index) const {
  return data[index];
}

template <typename T>
unsigned int FStack<T>::size() const {
  return m_size;
}

template <typename T>
unsigned int FStack<T>::capacity() const {
  return m_capacity;
}

template <typename T>
void FStack<T>::push(const T &value) {
  data[m_size++] = value;
}

template <typename T>
void FStack<T>::pop() {
  m_size--;
}

template <typename T>
T& FStack<T>::back() {
  return data[m_size-1];
}

template <typename T>
const T& FStack<T>::back() const {
  return data[m_size-1];
}

#endif
