#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <chrono>

#include "clparser.hpp"
#include "square.hpp"
#include "diagonals.hpp"
#include "solver.hpp"

using namespace std;

const string USAGE = "\
Usage:\n\
-h \t Print this string.\n\
-d \t Output using diagonals instead of full squares.\n\
-i \t Specify input file of templates.\n\
-o \t Specify output file.\n\
-s \t Specify which template to use from the input file.\
";

int main(int argc, char** argv) {
  argmap parsed;
  bool result = parseArgs(argc, argv, parsed);

  if (!result) {
    cerr << "Incorrect usage...\n\n" << argv[0] << "\n" << USAGE << endl;
    return 1;
  }

  if (parsed.count("-h")) {
    cout << argv[0] << "\n" << USAGE << endl;
    return 0;
  }

  bool diagonal_output = false;
  if (parsed.count("-d")) {
    // Ouput should be done using diagonals.
    diagonal_output = true;
  }

  std::ifstream istream;
  if (parsed.count("-i")) {
    istream = std::ifstream(parsed["-i"]);
    if (!istream.is_open()) {
      std::cerr << "Failed to open specified input file." << '\n';
      return 2;
    }
  }

  std::istream& input = (istream.is_open()) ? istream : std::cin;

  std::ofstream ostream;
  if (parsed.count("-o")) {
    ostream = std::ofstream(parsed["-o"]);
    if (!ostream.is_open()) {
      std::cerr << "Failed to open specified output file." << '\n';
      return 3;
    }
  }

  std::ostream& output = (ostream.is_open()) ? ostream : std::cout;

  unsigned int val1 = 0;
  if (parsed.count("-1")) {
    std::stringstream ss(parsed["-1"]);
    ss >> val1;
  }

  unsigned int val2 = 0;
  if (parsed.count("-2")) {
    std::stringstream ss(parsed["-2"]);
    ss >> val2;
  }

  std::string line;
  std::getline(input, line);
  std::stringstream ss(line);

  unsigned int count = 0;
  ss >> count;

  if (!parsed.count("-s")) {
    std::cerr << "You need to specify the square to deal with.\n";
    return 4;
  }

  ss = std::stringstream(parsed["-s"]);
  unsigned int s;
  ss >> s;

  const unsigned int lineLength = 21;
  input.seekg(s*(2 + 2*10*lineLength), std::ios::cur);
  std::vector<Square> squares = parseSquares(input);

  if (parsed.count("-1") && parsed.count("-2")) {
    swapSquares(squares, val1, val2);
  }

  DiagGenerator gen(squares);

  gen.generate();

  FArray<FArray<__uint128_t>> diagonals(2);
  diagonals[0] = FArray<__uint128_t>(gen.getSDiagonals().size());
  diagonals[1] = FArray<__uint128_t>(gen.getTDiagonals().size());
  for (unsigned int i = 0; i < diagonals[0].size(); i++) {
    diagonals[0][i] = gen.getSDiagonals()[i].low;
  }
  for (unsigned int i = 0; i < diagonals[1].size(); i++) {
    diagonals[1][i] = gen.getTDiagonals()[i].low;
  }

  std::chrono::steady_clock::time_point start = std::chrono::steady_clock::now();


  std::cout << s << " : " <<
    diagonals[0].size() << " : " << diagonals[1].size()<<"\n";

  unsigned int pairCount = solve(output, diagonals);

  std::cout << " : " <<
      std::chrono::duration_cast<std::chrono::seconds>(
        std::chrono::steady_clock::now() - start).count() << " : " << pairCount <<
    std::endl;

  output << "#END " << parsed["-i"] << " " << s << std::endl;

  return 0;
}
