// AUTHOR: Michael Gill
// School or Mathematics
// Monash University
// Clayton, Victoria, AU 3800
// email:  Michael.Gill@Monash.edu
//
// A set of program wide definitions and typename declarations. All header files
// should include this file to be ensure project consistency.
// Copyright (C) 2019  Michael Gill
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef LATIN10_DEFINITIONS_HPP_
#define LATIN10_DEFINITIONS_HPP_

#include <array>
#include <iostream>
#include <vector>

// Library wide Definitions
#define N 10
#define HALF_BYTE_S 4
#define HALF_BYTE_M 15
#define GDD_B_SIZES 6
#define MAX_SYMB (GDD_B_SIZES - 2)

// Integral typedefs
typedef unsigned __int128 uint128_t;

// Library wide typedefs
typedef unsigned char uchar_t;
typedef std::array<uchar_t, N * N> Square;
typedef std::array<uchar_t, N> Permutation;
// typedef std::array<std::array<bool, N>, MAX_SYMB> TRow;
// typedef std::array<std::array<bool, N*N>, MAX_SYMB> Template;

// Library wide exception with messages
class m_exception : public std::exception {
private:
  std::string message;

public:
  m_exception(std::string message) : message(message) {}

  virtual const char *what() const throw() { return message.c_str(); }
};

template <typename T>
std::ostream &operator<<(std::ostream &os, const std::vector<T> &vec) {
  os << "[";

  auto it = vec.begin();
  while (it + 1 != vec.end()) {
    os << *(it++) << ", ";
  }

  os << *it << "]";

  return os;
}

#endif // LATIN10_DEFINITIONS_HPP_
