// AUTHOR: Michael Gill
// School or Mathematics
// Monash University
// Clayton, Victoria, AU 3800
// email:  Michael.Gill@Monash.edu
//
// Functions for dealing with answers relating to transversals.
// Example: [Find the number of common transversals,
//           Find the largest set of common disjoint transversals]
// Copyright (C) 2019  Michael Gill
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef LATIN10_IO_CLOPTS_HPP_
#define LATIN10_IO_CLOPTS_HPP_

#include <map>
#include <string>
#include <any>
#include <numeric>
#include <vector>
#include <sstream>
#include <regex>

const std::string USAGE_HEADER = "Usage: [options] <positional args>\n";

enum TYPE {
  NONE,
  INTEGER,
  DOUBLE,
  STRING,
  OPTION
};

class CLOptions;

class CLOption {
private:
  std::string key;
  std::string description;
  TYPE t_def;
public:
  CLOption();
  CLOption(const std::string&, const std::string&, TYPE);
  TYPE getType();
  std::string getDesc();
};

class CLResults {
private:
  std::map<std::string, std::optional<std::any>> result_map;
  CLOptions& options;
  bool valid;
  std::vector<std::string> positional;
public:
  CLResults(CLOptions& options);
  std::optional<std::any> operator[](const char*);
  bool is_valid();
  operator bool ();

  friend class CLOptions;
};

class CLOptions {
private:
  std::map<std::string, std::string> key_map;
  std::map<std::string, CLOption> option_map;
  std::string name;
public:
  CLOptions();
  CLOptions(const std::string&);
  bool addOption(const std::string&, const std::string&);
  bool addOption(const std::string&, const std::string&, TYPE);
  CLResults parse(const int, char const **);
  const std::string& operator[](const std::string&);
  std::optional<CLOption> getOption(const std::string&);
  std::string usage();
};



#endif
