// AUTHOR: Michael Gill
// School or Mathematics
// Monash University
// Clayton, Victoria, AU 3800
// email:  Michael.Gill@Monash.edu
//
// Handles input of LS from istream. Decoder input specifications can be
// found in the manual.
// Copyright (C) 2019  Michael Gill
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef LATIN10_IO_DECODER_HPP_
#define LATIN10_IO_DECODER_HPP_

#include <iostream>


#include "definitions.hpp"
#include "number/integer.hpp"

// Base struct contract for decoder subclasses. The LS contained in the output
// stream is read into the square
struct Decoder {
  virtual void decode(std::istream& stream, Square& square) = 0;
  virtual ~Decoder();
};

// See manual for details on the numeric encoding.
struct NumericDecoder : public Decoder {
  virtual void decode(std::istream& stream, Square& square);
};

// See manual for details on the alpha encoding.
struct AlphaDecoder : public Decoder {
  virtual void decode(std::istream& stream, Square& square);
};

// See manual for details on the bitpacking encoding.
struct BitDecoder : public Decoder {
  virtual void decode(std::istream& stream, Square& square);
};

// See manual for details on the factorial encoding.
struct FactorialDecoder : public Decoder {
private:
  size_t minPos = 0;
public:
  virtual void decode(std::istream& stream, Square& square);
};

#endif // LATIN10_IO_DECODER_HPP_
