// AUTHOR: Michael Gill
// School or Mathematics
// Monash University
// Clayton, Victoria, AU 3800
// email:  Michael.Gill@Monash.edu
//
// Implementation of useful integer functions.
// Copyright (C) 2019  Michael Gill
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "number/integer.hpp"

size_t R_inv(const std::string& r, size_t blocks) {
  size_t val = 0;

  // Extract the 6 bit values from the chars and shift into place.
  for (unsigned int i = 0; i < blocks; i++) {
    val |= ((size_t)(r[i] - 63) & B_MASK) << ((blocks-(i+1))*BLOCK_S);
  }

  return val;
}

std::string R(size_t x, size_t blocks) {
  std::string coding(blocks, '0');

  // The first item represents the most sig block.
  for (unsigned int i = 0; i < blocks; i++) {
    coding[i] = (x >> (blocks-(i+1))*BLOCK_S) & B_MASK;
    coding[i] += 63;
  }

  return coding;
}

std::string pack(size_t n) {
  std::stringstream ss;

  // Condition on the specific storable ranges.
  if (n < 63) {
    ss << (char)(n + 63);
  } else if (n < 258048) {
    ss << (char)B_DELIM;

    std::string coding = R(n, MID);

    ss << coding;
  } else if (n < 68719476736) {
    ss << (char)B_DELIM << (char)B_DELIM;

    std::string coding = R(n, HIGH);

    ss << coding;
  } else {
    throw m_exception("Value too large to be packed...");
  }

  return ss.str();
}

size_t unpack(const std::string& r) {
  size_t count = 0;
  while (r[count] == B_DELIM) {
    count++;
  }

  if(count == 0) {
    return r[0] - 63;
  }

  size_t blocks = MID;
  if (count > 1) blocks = HIGH;

  return R_inv(r.c_str()+count, blocks);
}

const Permutation permFromPosition(size_t position) {
  Permutation result = {1,2,3,4,5,6,7,8,9,10};
  std::list<size_t> symbols(N);
  std::iota(symbols.begin(), symbols.end(), 1);

  for (size_t i = 0; i < N; i++) {
    size_t sym_skip = position/factorial[N-(i+1)];
    position %= factorial[N-(i+1)];

    auto sym_it = symbols.begin();
    while ((size_t)std::distance(symbols.begin(), sym_it) < sym_skip) {
      sym_it++;
    }
    result[i] = *sym_it;

    symbols.erase(sym_it);
  }

  return result;
}

unsigned int popCount(uint_fast64_t a) {
  unsigned int b = 0;

  while (a > 0) {
    b += (a & 1);
    a = a >> 1;
  }

  return b;
}

size_t hexctoi(const char c) {
  if ('0' <= c && c <= '9' ) {
    return (size_t)(c - '0');
  } else {
    if (c < 'a') {
      return (size_t)(c - 'A' + 10);
    } else {
      return (size_t)(c - 'a' + 10);
    }
  }
}

char itohexc(const size_t i) {
  if (i < 10) {
    return (char)(i + '0');
  } else {
    return (char)(i + 'A' - 10);
  }
}
